/* Licensed Materials - Property of IBM
 * 5724-L01
 * (C) Copyright IBM Corporation 2004. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package sel;

import java.util.List;
import com.ibm.websphere.sca.*;
import com.ibm.wbiserver.common.exception.*;
import com.ibm.wbiserver.common.selection.*;
import commonj.sdo.*;
import com.ibm.websphere.sca.scdl.OperationType;
import com.ibm.wbiservers.common.event.EventLogger;

public class SelectClean implements ServiceImplAsync, ServiceImplSync
{
    public static final String COPYRIGHT="(C) Copyright IBM Corporation 2004.";  //$NON-NLS-1$
    
    static final String SEL_QNAME  = "{http://www.ibm.com/xmlns/prod/websphere/wbi/SelectorComponentDef/6.0.0}SelectorComponentDef";
    static final String COMPONENT_TNS  = "http://AutoClean";
    static final String COMPONENT_NAME = "SelectClean";    
    static final String COMPONENT_TYPE = "SELT";    
	
    static EventLogger logger = new EventLogger(SEL_QNAME, COMPONENT_TNS, COMPONENT_NAME, 1);    
	    
    public Object invoke(OperationType operationType, Object input)
{
  logger.logEntry(operationType.getName(), input);

  Object result = null;

  try
  {
    result = invokeTarget(operationType, input);
  }
  catch (RuntimeException rex)
  {
    logger.logFailure(operationType.getName(), rex);
    throw rex;
  }

  logger.logExit(operationType.getName(), result);

  return result;
}
    
    public void invokeAsync(OperationType operationType, Object input, ServiceCallback callback, Ticket ticket)
{
  logger.logEntry(operationType.getName(), input);

  Object result = null;
  Exception returnException = null;

  try
  {
    result = invokeTarget(operationType, input);
  }
  catch (RuntimeException rex)
  {
    logger.logFailure(operationType.getName(), rex);
    returnException = rex;
  }

  logger.logExit(operationType.getName(), result);

  if (callback != null)
  {
    callback.onInvokeResponse(ticket, result, returnException);
  }
}
    
    Object invokeTarget(OperationType operationType, Object input)
{
  Object result = null;
  InvocationTarget invocationTarg = null;

  List targets = selectTargets(operationType, input);

  if (targets == null || targets.size() == 0)
  {
    throw new NoSelectionTargetException("No applications were found that meet the input selection criteria.");
  }

  // The InvocationTarget class knows how to invoke different types of subclasses.
  // Different types of subclasses invoke different types of targets.  In the case
  // of BusinessRules, the subclass is BusinessRuleInvocationTarget, and it knows
  // how to invoke BusinessRules.  In the case of SCA components, the subclass is
  // SCAInvocationTarget, and it knows how to invoke SCA components.

  try
  {
    invocationTarg = (InvocationTarget) targets.get(0);
  }
  catch (ClassCastException e)
  {
    throw new SelectorException("A custom SelectionAlgorithm must return a list of subclasses of the InvocationTarget class.");
  }

  logger.logTargetId(operationType.getName(), invocationTarg.getTargetID());

  result = invocationTarg.invoke(COMPONENT_TNS, COMPONENT_NAME, operationType, input);

  return result;
}
	
	List selectTargets(OperationType operationType, Object input)
	{
	   Object[] selectorParms = null;
	   SelectionAlgorithm selector = null;
	   String operationName = operationType.getName();
	   boolean inputIsAWrapper = operationType.isWrapperType(operationType.getInputType());
	   
	   // Each operation can define its own selection algorithm and selection parameters.
	   // The next section of code is a big switch statement over the operation name.  The
	   // result of this section of code is that the variables selectorParms will contain
	   // the selection parameters and selector will contain the SelectionAlgorithm for 
	   // the particular operation.
	   
	   try
	   {
		   // Setup a try/catch block to catch any ClassCastExceptions that 
		   // might be caused by a custom selector's failure to implement the
		   // SelectionAlgorithm interface.
	   
	       selector = (SelectionAlgorithm) com.ibm.wbiservers.common.selection.GenericSelector.getInstance();       
	        
	       if ("cleanseClip".equals(operationName))
	       {
		      selectorParms = getSelectionParmsForCleanseClip(input, inputIsAWrapper);
           }
           
		   else
		   {
		      throw new SelectorException("The operation " + operationName + " is not supported by this component.");
		   }
	   }
	   catch(ClassCastException e)
       {
           throw new InvalidSelectionAlgorithmException("Custom SelectionAlgorithms must implement the SelectionAlgorthm interface.");
       }
       
       logger.logSelectionKey(operationType.getName(), selectorParms);
       
        // Although the SelectionAlgorithm interface supports multicast, this component does not.
        // The third parameter tells the selection algorithm to return the first match only.
        // Therefore, the resulting List will contain either 0 or 1 entry.
        
	   return selector.selectTargets(COMPONENT_TNS, COMPONENT_NAME, COMPONENT_TYPE, operationName, selectorParms, false);
	}   	      
	    
    Object[] getSelectionParmsForCleanseClip(Object input, boolean inputIsAWrapper)
    {
		  
	   // Add all input parameters to the selectorParms Object array. 
	   // Use the XPaths and/or code snippets to mine the input object
	   // for the selection parameters.
	   
        // Since the interface for this operation indicates that the input should be wrapped,
 // we check the actual input to ensure that it is a wrapper type.  If it is not, an
 // exception will be thrown.

 if (inputIsAWrapper == false)
 {
    throw new SelectorException("The input for operation cleanseClip should be wrapped.");
 }
      
       Object[] tempParms = 
		  {  
			 getSelectionParm1ForCleanseClip(((DataObject) input).get("inClipBG")) 
		  };
		  		     
       return tempParms;    	
    }    
	    
    Object getSelectionParm1ForCleanseClip(Object inClipBG)
    {
        // User defined code snippet for operation cleanseClip to retrieve 
        // selection parameter number 1       
    	
        //@generated:com.ibm.wbit.selector.ui.CurrentDate
return new java.util.Date();
    }             
			 
 	   
 		 Object getXPathParmDefParameter(String param, String xPath, Object input, boolean isInputAWrapper)
		   {

		      boolean xPathSpecified = (xPath != null && xPath.equals("") == false);
		      boolean paramSpecified = (param != null && param.equals("") == false);

		      // catches all the cases where there is neither an xPath nor a param

		      if (xPathSpecified == false && paramSpecified == false)
		      {
		         return input;
		      }

		      // catches all the additional cases where there is no xPath

		      if (xPathSpecified == false && paramSpecified == true)
		      {
		         if (isInputAWrapper)
		         {
		            return ((DataObject)input).get(param);
		         }
		         else
		         {
		            return input;
		         }
		      }

		      // for all of the rest of the cases, we know there is an xPath

		      // divide the remaining cases up by whether the input is a wrapper type or not

		      if (isInputAWrapper)
		      {
		         if (paramSpecified == false)
		         {
		            throw new SelectorException("For wrapped input messages, the parameter attribute is required for the XPathParameterDef element.");
		         }
		         else
		         {
		            String xPathToUse = null;

		            if (xPath.startsWith("/") == true)
		            {
		               xPathToUse = param + xPath;
		            }
		            else
		            {
		               xPathToUse = param + "/" + xPath;
		            }

		            return ((DataObject)input).get(xPathToUse); 
		         }
		      }

		      // the input is not a wrapper type.  Since only xPath cases are left, throw an
		      // exception if the input is not a DataObject

		      if ((input instanceof DataObject) == false)
		      {
		         throw new SelectorException("The input message must contain a valid DataObject.");
		      }

		      return ((DataObject)input).get(xPath);
		   }
}